#!/bin/bash
#
#   USAGE:  build_gdb.sh <product_dir> [tar] [source]
#   NOTE:  we build gdb with the native compiler

usage()
{
   echo "USAGE: `basename ${0}` <product_dir> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`

    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

TMP=`mktemp -t build_gdb.sh.XXXXXX`
trap "rm $TMP* 2>/dev/null" EXIT

function maybe_setup_gcc() {
  [[ "$flvr" == Linux* ]] || return
  if ver_greater 4.8.0 $(gcc -dumpversion); then
    # Need special GCC that understands --static-stdlibc++ and
    # --static-libgcc.
    gccprod=$(latest_version -t ${product_dir} --ge 6.1.0 gcc -z "${product_dir}" -z "${PRODUCTS}")
    (( $? == 0 )) || \
      ssi_die "On systems with GCC < 4.8.0, need access to GCC >= 6.1. Best version found was ${best_version}"
    eval setup -B ${gccprod} || ssi_die "Unable to set up ${gccprod}"
  fi
  if ver_greater 2.17 $(/lib/libc.so.6 2>/dev/null | head -n 1 | sed -e 's&.*version \([^ ,]*\).*$&\1&'); then
    # See gdb-lrt.patch.
    extra_make_args=(RTLIB=-lrt)
  fi
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
maketar=${2}
sourcetar=${3}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=gdb
pkgver=v8_0_1
ssibuildshims_version=v1_02_00
pkgdotver=`echo ${pkgver} | sed -e 's/_/./g' -e 's/current/50/' | sed -e 's/^v//'`
starball=${product_dir}/${package}/${pkgver}/tar/gdb-${pkgdotver}.tar.gz

get_this_dir

get_ssibuildshims

if [ "${sourcetar}" = "source" ]
then
    ${SSIBUILDSHIMS_DIR}/bin/make_source_code_tarball ${product_dir} ${package} ${pkgver}
fi

source define_basics NO_QUAL NO_DEBUG_SOURCE

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/bin ]
then
    ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} NO_QUAL CURRENT
    exit 0
fi

echo "building ${package} for ${OS}-${plat} (flavor ${flvr}) against GCC ${GCC_VERSION}"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi
mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

cd ${sourcedir}
tar xf ${starball} || ssi_die "Unable to unwind ${starball}"

cd ${package}-${pkgdotver}

patches=(gdb-lrt.patch)
for patch in "${patches[@]}"; do
  patch -p1 ${DEBUG_PATCHES:+--verbose} -i ${patchdir}/${patch} || \
    ssi_die "Unable to apply patch ${patch}"
done

[[ -n "${DEBUG_PATCHES}" ]] && exit 1

cd ${blddir}

num_cores=$(${SSIBUILDSHIMS_DIR}/bin/ncores)

maybe_setup_gcc

ups active

set -x

${sourcedir}/${package}-${pkgdotver}/configure \
  --prefix=${pkgdir} \
  --with-python=no \
  || ssi_die "Unable to configure gdb."

make MAKE="make -j ${num_cores} ${extra_make_args[@]}" \
  "${extra_make_args[@]}" -j ${num_cores} all || ssi_die "Unable to make gdb."
make install  || ssi_die "Unable to install gdb."

set +x

if [ ! -e ${pkgdir}/bin/gdb ]
then
   echo "ERROR: the gdb build has failed"
   echo "       Something is very wrong"
   exit 1
fi

find "${pkgdir}" -name '*.la' | xargs rm -vf

${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} NO_QUAL CURRENT

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -z ${product_dir}
#which cmake

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/bin ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} NO_QUAL CURRENT
fi

exit 0
